<?php
    /*********************************************************************************
     * Zurmo is a customer relationship management program developed by
     * Zurmo, Inc. Copyright (C) 2017 Zurmo Inc.
     *
     * Zurmo is free software; you can redistribute it and/or modify it under
     * the terms of the GNU Affero General Public License version 3 as published by the
     * Free Software Foundation with the addition of the following permission added
     * to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED WORK
     * IN WHICH THE COPYRIGHT IS OWNED BY ZURMO, ZURMO DISCLAIMS THE WARRANTY
     * OF NON INFRINGEMENT OF THIRD PARTY RIGHTS.
     *
     * Zurmo is distributed in the hope that it will be useful, but WITHOUT
     * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
     * FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
     * details.
     *
     * You should have received a copy of the GNU Affero General Public License along with
     * this program; if not, see http://www.gnu.org/licenses or write to the Free
     * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
     * 02110-1301 USA.
     *
     * You can contact Zurmo, Inc. with a mailing address at 27 North Wacker Drive
     * Suite 370 Chicago, IL 60606. or at email address contact@zurmo.com.
     *
     * The interactive user interfaces in original and modified versions
     * of this program must display Appropriate Legal Notices, as required under
     * Section 5 of the GNU Affero General Public License version 3.
     *
     * In accordance with Section 7(b) of the GNU Affero General Public License version 3,
     * these Appropriate Legal Notices must retain the display of the Zurmo
     * logo and Zurmo copyright notice. If the display of the logo is not reasonably
     * feasible for technical reasons, the Appropriate Legal Notices must display the words
     * "Copyright Zurmo Inc. 2017. All rights reserved".
     ********************************************************************************/

    class AutorespondersPortletView extends MarketingListMembersPortletView
    {
        protected static $persistentUserPortletConfigs = array();

        public static function getDefaultMetadata()
        {
            $metadata = array(
                'global' => array(
                    'toolbar' => array(
                        'elements' => array(
                            array('type'              => 'AutoresponderCreateLink',
                                  'routeParameters'   => 'eval:$this->getAutoresponderCreateRouteParameters()',
                                  'redirectUrl'       => 'eval:$this->getNonAjaxRedirectUrl()',
                                  'pageVarName'       => 'eval:$this->getPageVarName()',
                                  'listViewGridId'    => 'eval:$this->getListGridId()'),
                        ),
                    ),
                ),
            );
            return $metadata;
        }

        public function renderPortletHeadContent()
        {
            return $this->renderViewToolBar();
        }

        public function getTitle()
        {
            return Zurmo::t('AutorespondersModule', 'Autoresponders');
        }

        protected function getConfigurationFormClassName()
        {
            return null;
        }

        protected function getListViewClassName()
        {
            return 'AutorespondersListView';
        }

        protected function getWrapperDivClass()
        {
            return MarketingListDetailsAndRelationsView::AUTORESPONDERS_PORTLET_CLASS;
        }

        protected function getListContentWrapperDivClass()
        {
            return 'marketing-list-autoresponders-list';
        }

        protected function getConfigUtilClassName()
        {
            return null;
        }

        protected function getSearchAttributes()
        {
            return  AutorespondersUtil::makeSearchAttributeData($this->modelId);
        }

        protected function getSortAttributes()
        {
            
            return array('Autoresponder' => "(CASE fromoperationdurationtype
                            WHEN 'Minute' THEN fromOperationDurationInterval
                            WHEN 'Hour' THEN fromOperationDurationInterval*60 
                            WHEN 'Day' THEN fromOperationDurationInterval*24*60 
                            WHEN 'Week' THEN fromOperationDurationInterval*7*24*60
                            WHEN 'Month' THEN fromOperationDurationInterval*30*24*60
                            WHEN 'Year' THEN fromOperationDurationInterval*365*24*60
                            END)");
        }

        protected function makeDataProvider()
        {
            $pageSize = Yii::app()->pagination->resolveActiveForCurrentUserByType('subListPageSize');
            $searchAttributes   = $this->getSearchAttributes();
            $sortAttributes     = $this->getSortAttributes();
            $sortDescending     = $this->getIsSortDescending();
            return new AutoresponderRedBeanModelsDataProvider($this->uniquePageId,
                                                    $sortAttributes,
                                                    $sortDescending,
                                                    $searchAttributes,
                                                    array('pagination' => array('pageSize' => $pageSize))
                                                );
        }
        
        protected function getIsSortDescending()
        {
            return AutorespondersUtil::getIsSortDescending();
        }

        protected function getAutoresponderCreateRouteParameters()
        {
            return array('marketingListId' => $this->modelId);
        }

        protected function shouldRenderActionElementBar()
        {
            return false;
        }
    }
?>