<?php
    /*********************************************************************************
     * Zurmo is a customer relationship management program developed by
     * Zurmo, Inc. Copyright (C) 2017 Zurmo Inc.
     *
     * Zurmo is free software; you can redistribute it and/or modify it under
     * the terms of the GNU Affero General Public License version 3 as published by the
     * Free Software Foundation with the addition of the following permission added
     * to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED WORK
     * IN WHICH THE COPYRIGHT IS OWNED BY ZURMO, ZURMO DISCLAIMS THE WARRANTY
     * OF NON INFRINGEMENT OF THIRD PARTY RIGHTS.
     *
     * Zurmo is distributed in the hope that it will be useful, but WITHOUT
     * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
     * FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
     * details.
     *
     * You should have received a copy of the GNU Affero General Public License along with
     * this program; if not, see http://www.gnu.org/licenses or write to the Free
     * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
     * 02110-1301 USA.
     *
     * You can contact Zurmo, Inc. with a mailing address at 27 North Wacker Drive
     * Suite 370 Chicago, IL 60606. or at email address contact@zurmo.com.
     *
     * The interactive user interfaces in original and modified versions
     * of this program must display Appropriate Legal Notices, as required under
     * Section 5 of the GNU Affero General Public License version 3.
     *
     * In accordance with Section 7(b) of the GNU Affero General Public License version 3,
     * these Appropriate Legal Notices must retain the display of the Zurmo
     * logo and Zurmo copyright notice. If the display of the logo is not reasonably
     * feasible for technical reasons, the Appropriate Legal Notices must display the words
     * "Copyright Zurmo Inc. 2017. All rights reserved".
     ********************************************************************************/

    /**
     * Sequential processing for the import data analysis.  Each step is a different column that is being mapped
     * for import.
     */
    class ImportDataAnalysisSequentialProcess extends ImportSequentialProcess
    {
        /**
         * A filtered down array of only column names that are mapped and analyzable.
         * @var array
         */
        protected $sanitizableColumnNames;

        protected static function resolveCustomFieldsInstructionDataIntoImportAndSaveImport(
                                  ImportDataAnalyzer $importDataAnalyzer, Import $import)
        {
            $adapter = new ImportToCustomFieldsInstructionDataAdapter($import);
            $adapter->appendCustomFieldsInstructionData($importDataAnalyzer->getCustomFieldsInstructionData());
            $saved   = $import->save();
            if (!$saved)
            {
                throw new FailedToSaveModelException();
            }
        }

        protected static function resolveSanitizableColumnNames($mappingData)
        {
            assert('is_array($mappingData)');
            $sanitizableColumnNames = array();
            foreach ($mappingData as $columnName => $columnMappingData)
            {
                if ($columnMappingData['attributeIndexOrDerivedType'] != null &&
                    $columnMappingData['type'] != 'extraColumn')
                {
                    $sanitizableColumnNames[] = $columnName;
                }
            }
            return $sanitizableColumnNames;
        }

        public function __construct(Import $import, ImportDataProvider $dataProvider)
        {
            $unserializedData             = unserialize($import->serializedData);
            $this->import                 = $import;
            $this->mappingData            = $unserializedData['mappingData'];
            $this->importRules            = ImportRulesUtil::makeImportRulesByType($unserializedData['importRulesType']);
            $this->dataProvider           = $dataProvider;
            $this->sanitizableColumnNames = static::resolveSanitizableColumnNames($this->mappingData);
        }

        public function getAllStepsMessage()
        {
            return Zurmo::t('ImportModule', 'Analyzing the import data...');
        }

        protected function steps()
        {
            return array('processRows', 'complete');
        }

        protected function stepMessages()
        {
            return array('processRows' => Zurmo::t('ImportModule', 'Analyzing'),
                         'complete'    => Zurmo::t('ImportModule', 'Finalizing Analysis...'));
        }

        protected function processRows($params)
        {
            $page = static::resolvePageByParams($params);
            $this->dataProvider->getPagination()->setCurrentPage($page);
            $importDataAnalyzer = new ImportDataAnalyzer($this->importRules, $this->dataProvider,
                                                         $this->mappingData, $this->sanitizableColumnNames);
            $importDataAnalyzer->analyzePage();
            static::resolveCustomFieldsInstructionDataIntoImportAndSaveImport($importDataAnalyzer, $this->import);
            return $this->resolveNextPagingAndParams($page, $params);
        }
        
        protected function resolveNextInformationByCurrentStep($step)
        {
            assert('is_string($step)');
            $steps        = $this->steps();
            $currentKey   = array_search($step, $steps);
            $nextKey      = $currentKey + 1;
            if (count($steps) == 2)
            {
                $this->completionPercentage = $this->subSequenceCompletionPercentage;
            }
            else
            {
                $extraPercentage = ($this->subSequenceCompletionPercentage / 100) * (100 / count($steps));
                $this->completionPercentage = ((($currentKey + 1) / count($steps)) * 100) + $extraPercentage;
            }
        }
    }
?>