<?php
    /*********************************************************************************
     * Zurmo is a customer relationship management program developed by
     * Zurmo, Inc. Copyright (C) 2017 Zurmo Inc.
     *
     * Zurmo is free software; you can redistribute it and/or modify it under
     * the terms of the GNU Affero General Public License version 3 as published by the
     * Free Software Foundation with the addition of the following permission added
     * to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED WORK
     * IN WHICH THE COPYRIGHT IS OWNED BY ZURMO, ZURMO DISCLAIMS THE WARRANTY
     * OF NON INFRINGEMENT OF THIRD PARTY RIGHTS.
     *
     * Zurmo is distributed in the hope that it will be useful, but WITHOUT
     * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
     * FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
     * details.
     *
     * You should have received a copy of the GNU Affero General Public License along with
     * this program; if not, see http://www.gnu.org/licenses or write to the Free
     * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
     * 02110-1301 USA.
     *
     * You can contact Zurmo, Inc. with a mailing address at 27 North Wacker Drive
     * Suite 370 Chicago, IL 60606. or at email address contact@zurmo.com.
     *
     * The interactive user interfaces in original and modified versions
     * of this program must display Appropriate Legal Notices, as required under
     * Section 5 of the GNU Affero General Public License version 3.
     *
     * In accordance with Section 7(b) of the GNU Affero General Public License version 3,
     * these Appropriate Legal Notices must retain the display of the Zurmo
     * logo and Zurmo copyright notice. If the display of the logo is not reasonably
     * feasible for technical reasons, the Appropriate Legal Notices must display the words
     * "Copyright Zurmo Inc. 2017. All rights reserved".
     ********************************************************************************/

    class Opportunity extends OwnedSecurableItem implements StarredInterface
    {
        public static function getByName($name)
        {
            return self::getByNameOrEquivalent('name', $name);
        }

        /**
         * @return value of what is considered the 'closed won' stage. It could be in the future named something else
         * or changed by the user.  This api will be expanded to handle that.  By default it will return 'Closed Won'
         */
        public static function getStageClosedWonValue()
        {
            return 'Closed Won';
        }

        public function onCreated()
        {
            parent::onCreated();
            $this->unrestrictedSet('stageModifiedDateTime',
                                    DateTimeUtil::convertTimestampToDbFormatDateTime(time()));
        }

        public function onModified()
        {
            parent::onModified();
            if (array_key_exists('value', $this->stage->originalAttributeValues) &&
                    $this->stage->originalAttributeValues['value'] != "" &&
                    $this->getScenario() != 'importModel')
            {
                $this->unrestrictedSet('stageModifiedDateTime',
                                    DateTimeUtil::convertTimestampToDbFormatDateTime(time()));
            }
            Yii::app()->custom->resolveOpportunityCustomActionsOnModified($this);
        }

        protected function beforeSave()
        {
            if (parent::beforeSave())
            {
                $automaticMappingDisabled = OpportunitiesModule::isAutomaticProbabilityMappingDisabled();
                if (!isset($this->originalAttributeValues['probability']) && $automaticMappingDisabled === false)
                {
                    $this->resolveStageToProbability();
                }
                $this->resolveAndSetDefaultSubscribers();
                Yii::app()->custom->resolveOpportunityCustomActionsBeforeSave($this);
                return true;
            }
            else
            {
                return false;
            }
        }

        protected function afterSave()
        {
            Yii::app()->custom->resolveOpportunityCustomActionsAfterSave($this);
            parent::afterSave();
        }

        protected function beforeDelete()
        {
            if (parent::beforeDelete())
            {
                Yii::app()->custom->resolveOpportunityCustomActionsBeforeDelete($this);
                return true;
            }
            return false;
        }

        public function __toString()
        {
            try
            {
                if (trim($this->name) == '')
                {
                    return Zurmo::t('Core', '(Unnamed)');
                }
                return $this->name;
            }
            catch (AccessDeniedSecurityException $e)
            {
                return '';
            }
        }

        public static function getModuleClassName()
        {
            return 'OpportunitiesModule';
        }

        public static function translatedAttributeLabels($language)
        {
            $params = LabelUtil::getTranslationParamsForAllModules();
            return array_merge(parent::translatedAttributeLabels($language), array(
                'account'     => Zurmo::t('AccountsModule',      'AccountsModuleSingularLabel', $params, null, $language),
                'amount'      => Zurmo::t('OpportunitiesModule', 'Amount',  array(), null, $language),
                'closeDate'   => Zurmo::t('OpportunitiesModule', 'Close Date',  array(), null, $language),
                'contacts'    => Zurmo::t('ContactsModule',      'ContactsModulePluralLabel',   $params, null, $language),
                'description' => Zurmo::t('ZurmoModule',         'Description',  array(), null, $language),
                'meetings'    => Zurmo::t('MeetingsModule',      'MeetingsModulePluralLabel', $params, null, $language),
                'name'        => Zurmo::t('Core',                'Name',  array(), null, $language),
                'notes'       => Zurmo::t('NotesModule',         'NotesModulePluralLabel', $params, null, $language),
                'probability' => Zurmo::t('OpportunitiesModule', 'Probability',  array(), null, $language),
                'source'      => Zurmo::t('ContactsModule',      'Source',   array(), null, $language),
                'stage'       => Zurmo::t('ZurmoModule',         'Stage',  array(), null, $language),
                'tasks'       => Zurmo::t('TasksModule',         'TasksModulePluralLabel', $params, null, $language),
                'stageModifiedDateTime'
                              => Zurmo::t('OpportunitiesModule', 'Stage Modified Date Time',  array(), null, $language)));
        }

        public static function canSaveMetadata()
        {
            return true;
        }

        public static function getDefaultMetadata()
        {
            $metadata = parent::getDefaultMetadata();
            $metadata[__CLASS__] = array(
                'members' => array(
                    'closeDate',
                    'description',
                    'name',
                    'probability',
                    'stageModifiedDateTime',
                ),
                'relations' => array(
                    'account'       => array(static::HAS_ONE,   'Account'),
                    'amount'        => array(static::HAS_ONE,   'CurrencyValue',    static::OWNED,
                                             static::LINK_TYPE_SPECIFIC, 'amount'),
                    'products'      => array(static::HAS_MANY, 'Product'),
                    'contacts'      => array(static::MANY_MANY, 'Contact'),
                    'stage'         => array(static::HAS_ONE,   'OwnedCustomField', static::OWNED,
                                             static::LINK_TYPE_SPECIFIC, 'stage'),
                    'source'        => array(static::HAS_ONE,   'OwnedCustomField', static::OWNED,
                                             static::LINK_TYPE_SPECIFIC, 'source'),
                    'projects'      => array(static::MANY_MANY, 'Project'),
                    'notificationSubscribers'   => array(static::HAS_MANY, 'NotificationSubscriber', static::OWNED,
                                             static::LINK_TYPE_POLYMORPHIC, 'relatedModel'),
                    'comments'      => array(static::HAS_MANY, 'Comment', static::OWNED,
                                             static::LINK_TYPE_POLYMORPHIC, 'relatedModel'),
                ),
                'derivedRelationsViaCastedUpModel' => array(
                    'meetings' => array(static::MANY_MANY, 'Meeting', 'activityItems'),
                    'notes'    => array(static::MANY_MANY, 'Note',    'activityItems'),
                    'tasks'    => array(static::MANY_MANY, 'Task',    'activityItems'),
                ),
                'rules' => array(
                    array('amount',                 'required'),
                    array('closeDate',              'required'),
                    array('closeDate',              'type',      'type' => 'date'),
                    array('description',            'type',      'type' => 'string'),
                    array('name',                   'required'),
                    array('name',                   'type',      'type' => 'string'),
                    array('name',                   'length',    'min'  => 1, 'max' => 64),
                    array('probability',            'type',      'type' => 'integer'),
                    array('probability',            'numerical', 'min' => 0, 'max' => 100),
                    array('probability',            'required'),
                    array('probability',            'default',   'value' => 0),
                    array('probability',            'probability'),
                    array('stage',                  'required'),
                    array('stageModifiedDateTime',  'required'),
                    array('stageModifiedDateTime',  'readOnly'),
                    array('stageModifiedDateTime',  'type', 'type' => 'datetime'),
                ),
                'elements' => array(
                    'amount'                => 'CurrencyValue',
                    'account'               => 'Account',
                    'closeDate'             => 'Date',
                    'description'           => 'TextArea',
                    'stageModifiedDateTime' => 'DateTime',
                ),
                'customFields' => array(
                    'stage'  => 'SalesStages',
                    'source' => 'LeadSources',
                ),
                'defaultSortAttribute' => 'name',
                'rollupRelations' => array(
                    'contacts',
                ),
                'noAudit' => array(
                    'description'
                ),
            );
            return $metadata;
        }

        public static function isTypeDeletable()
        {
            return true;
        }

        public static function isCacheable()
        {
            return Yii::app()->custom->resolveOpportunityIsCacheable();
        }

        public static function getRollUpRulesType()
        {
            return 'Opportunity';
        }

        public static function hasReadPermissionsOptimization()
        {
            return true;
        }

        public static function getGamificationRulesType()
        {
            return 'OpportunityGamification';
        }

        private function resolveStageToProbability()
        {
            if ($this->stage === null)
            {
                throw new NotSupportedException();
            }
            else
            {
                $this->probability = OpportunitiesModule::getProbabilityByStageValue($this->stage->value);
            }
        }

        /**
         * Override to handle the set read-only stageModifiedDateTime attribute on the import scenario.
         * (non-PHPdoc)
         * @see RedBeanModel::isAllowedToSetReadOnlyAttribute()
         */
        public function isAllowedToSetReadOnlyAttribute($attributeName)
        {
            if ($this->getScenario() == 'importModel' || $this->getScenario() == 'searchModel')
            {
                if ($attributeName == 'stageModifiedDateTime')
                {
                    return true;
                }
                else
                {
                    return parent::isAllowedToSetReadOnlyAttribute($attributeName);
                }
            }
        }

        /**
         * Resolve and set default subscribers
         */
        protected function resolveAndSetDefaultSubscribers()
        {
            NotificationSubscriberUtil::addSubscriber($this->owner, $this, false);
        }
    }
?>
