<?php
    /*********************************************************************************
     * Zurmo is a customer relationship management program developed by
     * Zurmo, Inc. Copyright (C) 2017 Zurmo Inc.
     *
     * Zurmo is free software; you can redistribute it and/or modify it under
     * the terms of the GNU Affero General Public License version 3 as published by the
     * Free Software Foundation with the addition of the following permission added
     * to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED WORK
     * IN WHICH THE COPYRIGHT IS OWNED BY ZURMO, ZURMO DISCLAIMS THE WARRANTY
     * OF NON INFRINGEMENT OF THIRD PARTY RIGHTS.
     *
     * Zurmo is distributed in the hope that it will be useful, but WITHOUT
     * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
     * FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
     * details.
     *
     * You should have received a copy of the GNU Affero General Public License along with
     * this program; if not, see http://www.gnu.org/licenses or write to the Free
     * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
     * 02110-1301 USA.
     *
     * You can contact Zurmo, Inc. with a mailing address at 27 North Wacker Drive
     * Suite 370 Chicago, IL 60606. or at email address contact@zurmo.com.
     *
     * The interactive user interfaces in original and modified versions
     * of this program must display Appropriate Legal Notices, as required under
     * Section 5 of the GNU Affero General Public License version 3.
     *
     * In accordance with Section 7(b) of the GNU Affero General Public License version 3,
     * these Appropriate Legal Notices must retain the display of the Zurmo
     * logo and Zurmo copyright notice. If the display of the logo is not reasonably
     * feasible for technical reasons, the Appropriate Legal Notices must display the words
     * "Copyright Zurmo Inc. 2017. All rights reserved".
     ********************************************************************************/

    /**
     * Helper class with functions to assist in working with Opportunities module.
     */
    class OpportunitiesUtil
    {
        /**
         * For the moment we keep 3 different methods for list views, related list views and kanban views.
         * Since a front end engineer has to do improvements it'll be more clear like this if the content
         * has to be slightly different. We could eventually merge into one, once the front end work is completed.
         *
         */

        public static function renderTimeInStageForListView($model)
        {
            $stageRottingValueInDays    = OpportunitiesModule::getRottingByStageValue($model->stage->value);
            if ($model->stageModifiedDateTime == '0000-00-00 00:00:00')
            {
                $timeInStageInDays          = null;
                $timeInStageDisplayContent  = Zurmo::t('OpportunitiesModule', 'Unknown');
            }
            else
            {
                $timeInStageInDays          = DateTimeUtil::getTimeSinceInDays($model->stageModifiedDateTime);
                $timeInStageDisplayContent  = DateTimeUtil::getTimeSinceDisplayContent($model->stageModifiedDateTime, true);
            }
            $span                       = ZurmoHtml::tag('span', array(), $timeInStageDisplayContent);

            if (OpportunitiesModule::isRottingMappingEnabled() && $stageRottingValueInDays != 0 && $timeInStageInDays !== null)
            {
                if ($timeInStageInDays < $stageRottingValueInDays)
                {
                    $class      = 'opportunity-time-stage completed';
                    $content    = '<i>&#9679;</i>' . $span;
                }
                else
                {
                    $class      = 'opportunity-time-stage paused';
                    $content    = '<i>&#9679;</i>' . $span;
                }
            }
            else
            {
                $class      = 'opportunity-time-stage';
                $content    = $span;
            }

            $content    = ZurmoHtml::tag('div', compact('class'), $content);
            return $content;
        }

        public static function renderTimeInStageForRelatedListView($model)
        {
            $stageRottingValueInDays    = OpportunitiesModule::getRottingByStageValue($model->stage->value);
            if ($model->stageModifiedDateTime == '0000-00-00 00:00:00')
            {
                $timeInStageInDays          = null;
                $timeInStageDisplayContent  = Zurmo::t('OpportunitiesModule', 'Unknown');
            }
            else
            {
                $timeInStageInDays          = DateTimeUtil::getTimeSinceInDays($model->stageModifiedDateTime);
                $timeInStageDisplayContent  = DateTimeUtil::getTimeSinceDisplayContent($model->stageModifiedDateTime, true);
            }
            $span                       = ZurmoHtml::tag('span', array(), $timeInStageDisplayContent);

            if (OpportunitiesModule::isRottingMappingEnabled() && $stageRottingValueInDays != 0 && $timeInStageInDays !== null)
            {
                if ($timeInStageInDays < $stageRottingValueInDays)
                {
                    $class      = 'opportunity-time-stage-related completed';
                    $content    = Zurmo::t('OpportunitiesModule', 'Time in stage') . ': ' . '<i>&#9679;</i>' . $span;
                }
                else
                {
                    $class      = 'opportunity-time-stage-related paused';
                    $content    = Zurmo::t('OpportunitiesModule', 'Time in stage') . ': ' . '<i>&#9679;</i>' . $span;
                }
            }
            else
            {
                $class      = 'opportunity-time-stage-related';
                $content    = Zurmo::t('OpportunitiesModule', 'Time in stage') . ': ' . $span;
            }

            $content    = ZurmoHtml::tag('div', compact('class'), $content);
            return $content;
        }

        public static function renderTimeInStageForKanbanView($model)
        {
            $stageRottingValueInDays    = OpportunitiesModule::getRottingByStageValue($model->stage->value);
            if ($model->stageModifiedDateTime == '0000-00-00 00:00:00')
            {
                $timeInStageInDays          = null;
                $timeInStageDisplayContent  = Zurmo::t('OpportunitiesModule', 'Unknown');
            }
            else
            {
                $timeInStageInDays          = DateTimeUtil::getTimeSinceInDays($model->stageModifiedDateTime);
                $timeInStageDisplayContent  = DateTimeUtil::getTimeSinceDisplayContent($model->stageModifiedDateTime, true);
            }

            if (OpportunitiesModule::isRottingMappingEnabled() && $stageRottingValueInDays != 0 && $timeInStageInDays !== null)
            {
                if ($timeInStageInDays < $stageRottingValueInDays)
                {
                    $class      = 'opportunity-time-stage-kanban completed';
                    $content    = Zurmo::t('OpportunitiesModule', 'Time in stage') . ': ' . '<i>&#9679;</i>' . $timeInStageDisplayContent;
                }
                else
                {
                    $class      = 'opportunity-time-stage-kanban paused';
                    $content    = Zurmo::t('OpportunitiesModule', 'Time in stage') . ': ' . '<i>&#9679;</i>' . $timeInStageDisplayContent;
                }
            }
            else
            {
                $class      = 'opportunity-time-stage-kanban';
                $content    = Zurmo::t('OpportunitiesModule', 'Time in stage') . ': ' . $timeInStageDisplayContent;
            }

            $content    = ZurmoHtml::tag('div', compact('class'), $content);
            return $content;
        }
    }
?>